/************************************************/
/*												*/
/*	definition file for IO-56 hardware V1.1 	*/
/*  fix bug in testprogram						*/
/*  add support for Nintendo flash cart         */
/*												*/
/************************************************/

int card_address;
BYTE card_data;
BYTE signals[] = { 0x00, 0x00, 0x00 };

	/* Configure 8255's 
	   B:
	   port A:  INPUT
	   port B:  INPUT
	   port Cl: OUTPUT   0: /WR
	   1: /RD
	   2: /MREQ
	   3: /WR_FLASH
	   port Ch: OUTPUT
	   A:
	   port A:  OUTPUT   ADDR 0..7
	   port B:  OUTPUT   ADDR 8..15
	   port Cl: INPUT    DATA 0..3
	   port Ch: INPUT    DATA 4..7
	 */

#define A_8255          (card_address)
#define A_8255_PA       (card_address)
#define A_8255_PB       (card_address+1)
#define A_8255_PC       (card_address+2)
#define A_8255_PROG     (card_address+3)
#define B_8255          (card_address+4)
#define B_8255_PA       (card_address+4)
#define B_8255_PB       (card_address+5)
#define B_8255_PC       (card_address+6)
#define B_8255_PROG     (card_address+7)

#define PA_in   		0x10
#define PA_out  		0x00
#define PB_in   		0x02
#define PB_out  		0x00
#define PCl_in  		0x01
#define PCl_out 		0x00
#define PCh_in  		0x08
#define PCh_out 		0x00

#define DATA_REG    	0x00
#define LOW_ADDR    	0x01
#define HIGH_ADDR   	0x02

#define SET_PC(bit)     (0x01|((bit&0x07)<<1))
#define CLR_PC(bit)     ((bit&0x07)<<1)

#define WR      		0
#define RD      		1
#define MREQ    		2
#define WR_FLASH   		3

#define LOAD(val)		signals[val] = card_data

#define WRITE(val)  	card_data = (val)

#define LED_OFF     	OUTPORT(B_8255_PC, 0xFF)

/* Read data from RAM(1) or ROM(0) */
BYTE READ(BYTE Mode)
{
  BYTE val;

  OUTPORT(A_8255_PA, signals[LOW_ADDR]);
  OUTPORT(A_8255_PB, signals[HIGH_ADDR]);
  if (Mode == RAM)
    OUTPORT(B_8255_PROG, CLR_PC(MREQ));
  OUTPORT(B_8255_PROG, CLR_PC(RD));
  val = INPORT(A_8255_PC);
  OUTPORT(B_8255_PROG, SET_PC(RD));
  if (Mode == RAM)
    OUTPORT(B_8255_PROG, SET_PC(MREQ));
  return (val);
}

/* Write data in bank-switch IC */
void WRITE_BKSW(data, address)
{
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_out | PCh_out);
  OUTPORT(A_8255_PC, (BYTE) data);
  OUTPORT(A_8255_PA, (BYTE) (address & 0x00FF));
  OUTPORT(A_8255_PB, (BYTE) (address >> 8));
  OUTPORT(B_8255_PROG, CLR_PC(WR));
  OUTPORT(B_8255_PROG, SET_PC(WR));
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_in | PCh_in);
}

/* Write data in RAM */
void WRITE_MEM(val)
{
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_out | PCh_out);
  OUTPORT(A_8255_PC, (BYTE) val);
  OUTPORT(A_8255_PA, signals[LOW_ADDR]);
  OUTPORT(A_8255_PB, signals[HIGH_ADDR]);
  OUTPORT(B_8255_PROG, CLR_PC(MREQ));
  OUTPORT(B_8255_PROG, CLR_PC(WR));
  OUTPORT(B_8255_PROG, SET_PC(WR));
  OUTPORT(B_8255_PROG, SET_PC(MREQ));
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_in | PCh_in);
}

/* Write data in Flash IC */
void WRITE_FLASH(val)
{
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_out | PCh_out);
  OUTPORT(A_8255_PC, (BYTE) val);
  OUTPORT(A_8255_PA, signals[LOW_ADDR]);
  OUTPORT(A_8255_PB, signals[HIGH_ADDR]);
  OUTPORT(B_8255_PROG, CLR_PC(WR_FLASH));
  OUTPORT(B_8255_PROG, SET_PC(WR_FLASH));
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_in | PCh_in);
}

/* Write data in Nintendo flash cart */
void WRITE_NFLASH(val)
{
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_out | PCh_out);
  OUTPORT(A_8255_PC, (BYTE) val);
  OUTPORT(A_8255_PA, signals[LOW_ADDR]);
  OUTPORT(A_8255_PB, signals[HIGH_ADDR]);
  OUTPORT(B_8255_PROG, CLR_PC(WR));
  OUTPORT(B_8255_PROG, SET_PC(WR));
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_in | PCh_in);
}

/* write low address byte */
void ADDRESS_LOW(val)
{
  signals[LOW_ADDR] = (BYTE) val;
}

/* write high address byte */
void ADDRESS_HIGH(val)
{
  signals[HIGH_ADDR] = (BYTE) val;
}

/* usage of hardware */
void usage_hardware(void)
{
  printf
    ("\t-l\tSpecify the IO-56 cardaddress (default is 1 = 0x300, 0 = 0x310)\n");
}

/* init hardware */
void init_port(int port)
{
  if (port == 1) {
    card_address = 0x300;
  } else {
    card_address = 0x310;
  }
  if (port == 0)
    printf("Address of IO-56 programming card = %hx hex\n", card_address);
  getPermissions(card_address, 8);
  OUTPORT(B_8255_PROG, 0x80 | PA_in | PB_in | PCl_out | PCh_out);
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_in | PCh_in);
  OUTPORT(B_8255_PC, 0xFF);
}

/* testprogram for IO-56 */
void test()
{
  printf("\nD0 - D7 data lines check !\n");
  printf("Check pins <22 up to 29> of GB connector\n");
  OUTPORT(A_8255_PROG, 0x80 | PA_out | PB_out | PCl_out | PCh_out);
  OUTPORT(A_8255_PC, 0x00);
  OUTPORT(A_8255_PA, 0x00);
  OUTPORT(A_8255_PB, 0x00);
  printf("They should all be low. Press <RETURN>\n");
  getch();

  OUTPORT(A_8255_PC, 0xFF);
  printf("Now, they should all be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nA0 - A7 address lines check !\n");
  printf("Check pins <6 up to 13> of GB connector\n");
  OUTPORT(A_8255_PA, 0x00);
  printf("They should all be low. Press <RETURN>\n");
  getch();

  OUTPORT(A_8255_PA, 0xFF);
  printf("Now, they should all be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nA8 - A15 address lines check !\n");
  printf("Check pins <14 up to 21> of GB connector\n");
  OUTPORT(A_8255_PB, 0x00);
  printf("They should all be low. Press <RETURN>\n");
  getch();

  OUTPORT(A_8255_PB, 0xFF);
  printf("Now, they should all be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nCS line check !\n");
  printf("check pin <5> of GB connector\n");
  OUTPORT(B_8255_PROG, CLR_PC(MREQ));
  printf("Should be low. Press <RETURN>\n");
  getch();

  OUTPORT(B_8255_PROG, SET_PC(MREQ));
  printf("Should be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nRD line check !\n");
  printf("check pin <4> of GB connector\n");
  OUTPORT(B_8255_PROG, CLR_PC(RD));
  printf("Should be low. Press <RETURN>\n");
  getch();

  OUTPORT(B_8255_PROG, SET_PC(RD));
  printf("Should be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nWR line check !\n");
  printf("check pin <3> of GB connector\n");
  OUTPORT(B_8255_PROG, CLR_PC(WR));
  printf("Should be low. Press <RETURN>\n");
  getch();

  OUTPORT(B_8255_PROG, SET_PC(WR));
  printf("Should be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nAudio IN line check !\n");
  printf("check pin <31> of GB connector\n");
  OUTPORT(B_8255_PROG, CLR_PC(WR_FLASH));
  printf("Should be low. Press <RETURN>\n");
  getch();

  OUTPORT(B_8255_PROG, SET_PC(WR_FLASH));
  printf("Should be high. Press <RETURN>\n");
  getch();

  LED_OFF;
  printf("\n\n\nTest complete !\n\n");
}
